@@ -2,7 +2,7 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "2.34_01";
+our $VERSION = "2.48_01";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -15,6 +15,8 @@ use Config;
 use Carp;
 use Try::Tiny;
 use Encode qw();
+use URI;
+use URI::redis;
 
 =head1 NAME
 
@@ -32,8 +34,7 @@ RedisDB - Perl extension to access redis database
 
 This module provides interface to access redis key-value store, it
 transparently handles disconnects and forks, supports transactions,
-pipelining, and subscription mode. Module includes XS and pure Perl
-versions of the parser.
+pipelining, and subscription mode.
 
 =head1 METHODS
 
@@ -41,7 +42,7 @@ versions of the parser.
 
 =head2 $class->new(%options)
 
-Creates the new RedisDB object. The following options are accepted:
+Creates a new RedisDB object. The following options are accepted:
 
 =over 4
 
@@ -61,7 +62,7 @@ L</host> and L</port> you should specify I<path>.
 =item password
 
 Password, if redis server requires authentication. Alternatively you can use
-I<auth> method after connection.
+I<auth> method after creating the object.
 
 =item database
 
@@ -70,21 +71,28 @@ connecting to the server. Database changes when you sending I<select> command
 to the server. You can get current database using I<selected_database> method.
 Default value is 0.
 
+=item url
+
+A Redis URL as described in L<URI::redis>.
+
+You cannot use C<url> together with any of C<host>, C<port>, C<path>,
+C<password>, C<database>.
+
 =item raise_error
 
 By default if redis-server returned error reply, or there was a connection
-error I<get_reply> method throws an exception of L<RedisDB::Error> type, if
-you set this parameter to false it will return an error object instead. Note,
-that if you have set this to false you must always check if the result you've
-got from RedisDB is a L<RedisDB::Error> object.
+error I<get_reply> method throws an exception of L<RedisDB::Error> type, if you
+set this parameter to false it will return an error object instead. Note, that
+if you set this to false you should always check if the result you've got from
+RedisDB method is a L<RedisDB::Error> object.
 
 =item timeout
 
-IO timeout. With this option set, if IO operation has taken more than specified
-number of seconds, module will croak or return L<RedisDB::Error::EAGAIN> error
-object depending on L</raise_error> setting. Note, that some OSes do not
-support SO_RCVTIMEO, and SO_SNDTIMEO socket options, in this case timeout will
-not work.
+IO timeout. With this option set, if I/O operation has taken more than
+specified number of seconds, module will croak or return
+L<RedisDB::Error::EAGAIN> error object depending on L</raise_error> setting.
+Note, that some OSes do not support SO_RCVTIMEO, and SO_SNDTIMEO socket
+options, in this case timeout will not work.
 
 =item utf8
 
@@ -94,39 +102,39 @@ from UTF-8. See L</"UTF-8 SUPPORT">.
 
 =item connection_name
 
-After establishing connection set its name. See "CLIENT SETNAME" command
-description in the redis documentation.
+After establishing a connection set its name to the specified using "CLIENT
+SETNAME" command.
 
 =item lazy
 
-by default I<new> establishes connection to the server. If this parameter is
-set, then connection will be established when you will send a command to the
-server.
+by default I<new> establishes a connection to the server. If this parameter is
+set, then connection will be established only when you will send first command
+to the server.
 
 =item reconnect_attempts
 
 this parameter allows you to specify how many attempts to (re)connect to the
-server should be made before returning error. Default value is 1, set to -1 if
-module should try to reconnect indefinitely.
+server should be made before returning an error. Default value is 1, set to -1
+if module should try to reconnect indefinitely.
 
 =item reconnect_delay_max
 
-module makes a delay before each new attempt to connect. Delay increases with
-each new attempt. This parameter allows you to specify maximum delay between
-attempts to reconnect. Default value is 10.
+module waits some time before every new attempt to connect. Delay increases
+each time. This parameter allows you to specify maximum delay between attempts
+to reconnect. Default value is 10.
 
 =item on_connect_error
 
-this allows you to specify callback that will be invoked if module could not
-establish connection to the server.  First argument to callback is a reference
-to the RedisDB object, and second is the error description. You must not invoke
-any methods on the object inside the callback, but you can change I<port> and
-I<host>, or I<path> attributes of the I<RedisDB> object to point to another
-server.  After callback returned, module tries to establish connection again
-using new parameters. To prevent further connection attempts callback should
-throw an exception. Default callback confesses. This may be useful to switch to
-backup server if primary went down. RedisDB distribution includes an example of
-using this callback in eg/server_failover.pl.
+if module failed to establish connection with the server it will invoke this
+callback.  First argument to the callback is a reference to the RedisDB object,
+and second is the error description. You must not invoke any methods on the
+object inside the callback, but you can change I<port> and I<host>, or I<path>
+attributes of the I<RedisDB> object to point to another server.  After callback
+returned, module tries to establish connection again using new parameters. To
+prevent further connection attempts callback should throw an exception, which
+is done by default callback. This may be useful to switch to backup server if
+primary went down. RedisDB distribution includes an example of using this
+callback in eg/server_failover.pl.
 
 =back
 
@@ -139,6 +147,13 @@ sub new {
     if ( $self->{path} and ( $self->{host} or $self->{port} ) ) {
         croak "You can't specify \"path\" together with \"host\" and \"port\"";
     }
+    if ( $self->{url} ) {
+        if ( $self->{host} or $self->{port} or $self->{path} ) {
+            croak "You can't specify \"url\" together with \"host\", \"port\" and \"path\"";
+        }
+
+        $self->_parse_url( $self->{url} );
+    }
     $self->{port} ||= 6379;
     $self->{host} ||= 'localhost';
     $self->{raise_error}    = 1 unless exists $self->{raise_error};
@@ -153,6 +168,26 @@ sub new {
     return $self;
 }
 
+sub _parse_url {
+    my ($self, $url) = @_;
+
+    my $uri = URI->new($url);
+
+    if ( $uri->scheme !~ /^redis/ ) {
+        die "Unknown URL scheme '" . $uri->scheme . "' in URL '$url'";
+    }
+
+    $self->{host}     = $uri->host;
+    $self->{port}     = $uri->port;
+    $self->{path}     = $uri->socket_path;
+    $self->{password} = $uri->password;
+    $self->{database} = $uri->database;
+}
+
+sub _is_redisdb_error {
+    ref(shift) =~ /^RedisDB::Error/;
+}
+
 sub _init_parser {
     my $self = shift;
     $self->{_parser} = RedisDB::Parser->new(
@@ -164,10 +199,11 @@ sub _init_parser {
 
 =head2 $self->execute($command, @arguments)
 
-send a command to the server and return the result. It will throw an exception
-if the server returns an error or return L<RedisDB::Error> depending on
-L</raise_error> parameter. It may be more convenient to use instead of this
-method wrapper named after the corresponding redis command. E.g.:
+send a command to the server, wait for the result and return it. It will throw
+an exception if the server returns an error or return L<RedisDB::Error> object
+depending on L</raise_error> parameter. It may be more convenient to use
+instead of this method wrapper named after the corresponding redis command.
+E.g.:
 
     $redis->execute('set', key => 'value');
     # is the same as
@@ -175,9 +211,9 @@ method wrapper named after the corresponding redis command. E.g.:
 
 See L</"WRAPPER METHODS"> section for the full list of defined aliases.
 
-Note, that you can't use I<execute> if you have sent some commands using
-I<send_command> method without callback argument and have not yet got all
-replies.
+Note, that you can not use I<execute> if you have sent some commands using
+I<send_command> method without the I<callback> argument and have not yet got
+all replies.
 
 =cut
 
@@ -205,7 +241,7 @@ sub _on_disconnect {
     if ($err) {
         $error_obj ||= RedisDB::Error::DISCONNECTED->new(
             "Server unexpectedly closed connection. Some data might have been lost.");
-        if ( $self->{raise_error} || $self->{_in_multi} ) {
+        if ( $self->{raise_error} or $self->{_in_multi} or $self->{_watching} ) {
             $self->reset_connection;
             die $error_obj;
         }
@@ -356,7 +392,7 @@ sub _connect {
             $self->{password},
             sub {
                 my ( $self, $res ) = @_;
-                croak "$res" if ref $res eq 'RedisDB::Error';
+                croak "$res" if _is_redisdb_error($res);
             }
         );
     }
@@ -413,7 +449,7 @@ sub _recv_data_nb {
         else {
             delete $self->{_socket};
 
-            if ( $self->{_parser}->callbacks or $self->{_in_multi} ) {
+            if ( $self->{_parser}->callbacks or $self->{_in_multi} or $self->{_watching} ) {
 
                 # there are some replies lost
                 $self->_on_disconnect(1);
@@ -446,13 +482,14 @@ sub _queue {
 
 send a command to the server. If send has failed command will die or return
 L<RedisDB::Error> object depending on L<raise_error> parameter.  Note, that it
-does not return reply from the server, you should retrieve it using the
-I<get_reply> method, or if I<callback> has been specified, it will be invoked
-upon receiving the reply with two arguments: the RedisDB object, and the reply
-from the server.  If the server returns an error, the second argument to the
-callback will be a L<RedisDB::Error> object, you can get description of the
-error using this object in string context.  If you are not interested in reply,
-you can use RedisDB::IGNORE_REPLY constant as the last argument.
+does not return reply from the server, if I<callback> was not specified, you
+should retrieve result using I<get_reply> method, otherwise I<callback>  will
+be invoked upon receiving the result with two arguments: the RedisDB object,
+and the reply from the server. If the server returned an error, the second
+argument to the callback will be a L<RedisDB::Error> object, you can get
+description of the error using this object in string context.  If you are not
+interested in reply, you can use RedisDB::IGNORE_REPLY constant as the last
+argument.
 
 Note, that RedisDB does not run any background threads, so it will not receive
 the reply and invoke the callback unless you call some of its methods which
@@ -534,7 +571,7 @@ sub send_command {
 
 sub _ignore {
     my ( $self, $res ) = @_;
-    if ( ref $res eq 'RedisDB::Error' ) {
+    if ( _is_redisdb_error($res) ) {
         warn "Ignoring error returned by redis-server: $res";
     }
 }
@@ -585,10 +622,9 @@ sub send_command_cb {
 
 =head2 $self->reply_ready
 
-This method may be used in the pipelining mode to check if there are some
-replies already received from the server. Returns the number of replies
-available for reading. May also return L<RedisDB::Error> object if there was a
-network error and I<raise_error> is disabled.
+this method may be used in the pipelining mode to check if there are some
+replies already received from the server. Returns true if there are replies
+ready to be fetched with I<get_reply> method.
 
 =cut
 
@@ -649,9 +685,9 @@ sub mainloop {
 
 =head2 $self->get_reply
 
-Receive and return reply from the server. If the server returned an error,
+receive and return reply from the server. If the server returned an error,
 method throws L<RedisDB::Error> exception or returns L<RedisDB::Error> object,
-depending on I<raise_error> parameter, see I<new>.
+depending on the L</raise_error> parameter.
 
 =cut
 
@@ -689,8 +725,8 @@ sub get_reply {
     }
 
     my $res = shift @{ $self->{_replies} };
-    if ( ref $res eq 'RedisDB::Error'
-        and ( $self->{raise_error} or $self->{_in_multi} ) )
+    if ( _is_redisdb_error($res)
+        and ( $self->{raise_error} or $self->{_in_multi} or $self->{_watching} ) )
     {
         croak $res;
     }
@@ -719,9 +755,9 @@ sub get_reply {
 
 =head2 $self->get_all_replies
 
-Wait till replies to all the commands without callback set will be received.
-Return a list of replies to these commands. For commands with callback set
-replies are processed as usual. Unlike I<mainloop> this method block only till
+wait till replies to all the commands without callback set will be received.
+Returns a list of replies to these commands. For commands with callback set
+replies are processed as usual. Unlike I<mainloop> this method blocks only till
 replies to all commands for which callback was NOT set will be received.
 
 =cut
@@ -737,7 +773,7 @@ sub get_all_replies {
 
 =head2 $self->replies_to_fetch
 
-Return the number of commands sent to the server replies to which wasn't yet
+return the number of commands sent to the server replies to which were not yet
 retrieved with I<get_reply> or I<get_all_replies>. This number only includes
 commands for which callback was not set.
 
@@ -750,7 +786,7 @@ sub replies_to_fetch {
 
 =head2 $self->selected_database
 
-Get currently selected database.
+get currently selected database.
 
 =cut
 
@@ -760,9 +796,9 @@ sub selected_database {
 
 =head2 $self->reset_connection
 
-Resets connection. This closes existing connection and drops all previously
-sent requests. After invoking this method the object returns to the same state
-as it was returned by the constructor.
+reset connection. This method closes existing connection and drops all
+previously sent requests. After invoking this method the object returns to the
+same state as it was returned by the constructor.
 
 =cut
 
@@ -777,7 +813,7 @@ sub reset_connection {
 
 =head2 $self->version
 
-Return the version of the server the client is connected to. The version is
+return the version of the server the client is connected to. The version is
 returned as a floating point number represented the same way as the perl
 versions. E.g. for redis 2.1.12 it will return 2.001012.
 
@@ -794,12 +830,14 @@ sub version {
 
 # don't forget to update POD
 my @commands = qw(
-  append	auth	bgrewriteaof	bgsave	bitcount	bitop	bitpos
+  append	asking	auth	bgrewriteaof	bgsave	bitcount	bitop	bitpos
   blpop	brpop   brpoplpush	client	client_kill	client_getname	client_setname
+  cluster	command
   config	config_get	config_set	config_resetstat	config_rewrite
   dbsize	debug_error	debug_object	debug_segfault
   decr	decrby	del	dump	echo	eval    evalsha exists	expire	expireat	flushall
-  flushdb	get	getbit	getrange	getset	hdel	hexists	hget	hgetall
+  flushdb	geoadd	geodist	geohash	geopos	georadius	georadiusbymember
+  get	getbit	getrange	getset	hdel	hexists	hget	hgetall
   hincrby	hincrbyfloat	hkeys	hlen	hmget	hscan	hmset	hset	hsetnx	hvals	incr	incrby
   incrbyfloat	keys	lastsave	lindex	linsert	llen	lpop	lpush	lpushx
   lrange	lrem	lset	ltrim	mget	migrate	move	mset	msetnx	object	object_refcount
@@ -810,7 +848,7 @@ my @commands = qw(
   script_load   sdiff	sdiffstore	select	set
   setbit	setex	setnx	setrange	sinter	sinterstore
   sismember	slaveof	slowlog smembers	smove	sort	spop	srandmember
-  srem	sscan	strlen	sunion	sunionstore	sync	time    ttl	type	unwatch watch
+  srem	sscan	strlen	sunion	sunionstore	time    ttl	type
   zadd	zcard	zcount	zincrby	zinterstore	zlexcount	zrange	zrangebylex
   zrangebyscore	zrank	zrem	zremrangebylex
   zremrangebyrank   zremrangebyscore	zrevrange	zrevrangebyscore	zrevrank
@@ -840,12 +878,13 @@ I<execute>, waits for the reply from the server, and returns it. E.g.:
     # equivalent to
     $redis->send_command("get", $key, sub { $val = $_[1] });
 
-The following wrapper methods are defined: append, auth, bgrewriteaof, bgsave,
+The following wrapper methods are defined: append, asking, auth, bgrewriteaof, bgsave,
 bitcount, bitop, bitpos, blpop, brpop, brpoplpush, client, client_kill,
-client_getname, client_setname, config, config_get, config_set,
+client_getname, client_setname, cluster, command, config, config_get, config_set,
 config_resetstat, config_rewrite, dbsize, debug_error, debug_object, debug_segfault, decr,
 decrby, del, dump, echo, eval, evalsha, exists, expire, expireat, flushall,
-flushdb, get, getbit, getrange, getset, hdel, hexists, hget, hgetall, hincrby,
+flushdb, geoadd, geodist, geohash, geopos, georadius, georadiusbymember,
+get, getbit, getrange, getset, hdel, hexists, hget, hgetall, hincrby,
 hincrbyfloat, hkeys, hlen, hmget, hscan, hmset, hset, hsetnx, hvals, incr,
 incrby, incrbyfloat, keys, lastsave, lindex, linsert, llen, lpop, lpush,
 lpushx, lrange, lrem, lset, ltrim, mget, migrate, move, mset, msetnx, object,
@@ -855,7 +894,7 @@ pubsub_numpat, quit, randomkey, rename, renamenx, restore, rpop, rpoplpush,
 rpush, rpushx, sadd, save, scan, scard, script, script_exists, script_flush,
 script_kill, script_load, sdiff, sdiffstore, select, set, setbit, setex, setnx,
 setrange, sinter, sinterstore, sismember, slaveof, slowlog, smembers, smove,
-sort, spop, srandmember, srem, sscan strlen, sunion, sunionstore, sync, time,
+sort, spop, srandmember, srem, sscan strlen, sunion, sunionstore, time,
 ttl, type, unwatch, watch, zadd, zcard, zcount, zincrby, zinterstore,
 zlexcount, zrange, zrangebylex, zrangebyscore, zrank, zrem, zremrangebylex,
 zremrangebyrank, zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
@@ -886,7 +925,26 @@ The following commands implement some additional postprocessing of the results:
 
 =cut
 
-=head2 $self->info([$callback])
+sub _execute_with_postprocess {
+    my $self  = shift;
+    my $ppsub = pop;
+    if ( $_[-1] && ref $_[-1] eq 'CODE' ) {
+        my $orig = pop;
+        my $cb   = sub {
+            my ( $redis, $reply ) = @_;
+            $reply = $ppsub->($reply) unless _is_redisdb_error($reply);
+            $orig->( $redis, $reply );
+        };
+        return $self->send_command( @_, $cb );
+    }
+    else {
+        my $reply = $self->execute(@_);
+        $reply = $ppsub->($reply) unless _is_redisdb_error($reply);
+        return $reply;
+    }
+}
+
+=head2 $self->info([\&callback])
 
 return information and statistics about the server. Redis-server returns
 information in form of I<field:value>, the I<info> method parses result and
@@ -896,18 +954,7 @@ returns it as a hash reference.
 
 sub info {
     my $self = shift;
-    my $orig = $_[-1];
-    if ( $orig && ref $orig eq 'CODE' ) {
-        my $cb = sub {
-            my ( $redis, $info ) = @_;
-            $orig->( $redis, _parse_info($info) );
-        };
-        return $self->send_command( 'INFO', $cb );
-    }
-    else {
-        my $info = $self->execute('INFO');
-        return _parse_info($info);
-    }
+    return $self->_execute_with_postprocess('INFO', @_, \&_parse_info);
 }
 
 sub _parse_info {
@@ -917,7 +964,7 @@ sub _parse_info {
     return \%info;
 }
 
-=head2 $self->client_list([$callback])
+=head2 $self->client_list([\&callback])
 
 return list of clients connected to the server. This method parses server
 output and returns result as reference to array of hashes.
@@ -926,18 +973,7 @@ output and returns result as reference to array of hashes.
 
 sub client_list {
     my $self = shift;
-    my $orig = $_[-1];
-    if ( $orig && ref $orig eq 'CODE' ) {
-        my $cb = sub {
-            my ( $redis, $list ) = @_;
-            $orig->( $redis, _parse_client_list($list) );
-        };
-        return $self->send_command( qw(CLIENT LIST), $cb );
-    }
-    else {
-        my $list = $self->execute(qw(CLIENT LIST));
-        return _parse_client_list($list);
-    }
+    return $self->_execute_with_postprocess('CLIENT', 'LIST', @_, \&_parse_client_list);
 }
 
 sub _parse_client_list {
@@ -952,6 +988,114 @@ sub _parse_client_list {
     return $res;
 }
 
+=head2 $self->cluster_info([\&callback])
+
+return information and statistics about the cluster. Redis-server returns
+information in form of I<field:value>, the I<cluster_info> method parses result
+and returns it as a hash reference.
+
+=cut
+
+sub cluster_info {
+    my $self = shift;
+    return $self->_execute_with_postprocess('CLUSTER', 'INFO', @_, \&_parse_info);
+}
+
+=head2 $self->cluster_nodes([\&callback])
+
+return list of cluster nodes. Each node represented as a hash with the
+following keys: node_id, address, host, port, flags, master_id, last_ping_sent,
+last_pong_received, link_state, slots.
+
+=cut
+
+sub cluster_nodes {
+    my $self = shift;
+    return $self->_execute_with_postprocess( 'CLUSTER', 'NODES', @_,
+        sub { $self->_parse_cluster_nodes(@_) } );
+}
+
+sub _parse_cluster_nodes {
+    my ($self, $list) = @_;
+
+    my @nodes;
+    for ( split /^/, $list ) {
+        my ( $node_id, $addr, $flags, $master_id, $ping, $pong, $state, @slots ) =
+          split / /;
+        my %flags = map { $_ => 1 } split /,/, $flags;
+        my ( $host, $port ) = split /:([^:]+)$/, $addr;
+        unless ($host) {
+            $host = $self->{host}, $addr = "$self->{host}:$port",
+        }
+        my $node = {
+            node_id            => $node_id,
+            address            => $addr,
+            host               => $host,
+            port               => $port,
+            flags              => \%flags,
+            master_id          => $master_id,
+            last_ping_sent     => $ping,
+            last_pong_received => $pong,
+            link_state         => $state,
+            slots              => \@slots,
+        };
+        push @nodes, $node;
+    }
+
+    return \@nodes;
+}
+
+sub _parse_role {
+    my $role = shift;
+
+    my $parsed = {
+        role => $role->[0],
+    };
+    if ( $parsed->{role} eq 'master' ) {
+        $parsed->{replication_offset} = $role->[1];
+        for ( @{ $role->[2] } ) {
+            push @{ $parsed->{slaves} },
+              {
+                host               => $_->[0],
+                port               => $_->[1],
+                replication_offset => $_->[2],
+              };
+        }
+    }
+    elsif ( $parsed->{role} eq 'slave' ) {
+        $parsed->{master} = {
+            host => $role->[1],
+            port => $role->[2],
+        };
+        $parsed->{status}             = $role->[3];
+        $parsed->{replication_offset} = $role->[4];
+    }
+    elsif ( $parsed->{role} eq 'sentinel' ) {
+        for ( @{ $role->[1] } ) {
+            push @{ $parsed->{services} }, $_;
+        }
+    } else {
+        confess "Unknown role $parsed->{role}";
+    }
+
+    return $parsed;
+}
+
+=head2 $self->role([\&callback])
+
+return reference to a hash describing the role of the server. Hash contains
+"role" element that can be either "master", "slave", or "sentinel". For master
+hash will also contain "replication_offset" and "slaves" elements, for slave it
+will contain "master", "status", and "replication_offset" elements, and for
+sentinel it will contain "services".
+
+=cut
+
+sub role {
+    my $self = shift;
+    return $self->_execute_with_postprocess( 'ROLE', @_, \&_parse_role );
+}
+
 =head2 $self->shutdown
 
 Shuts the redis server down. Returns undef, as the server doesn't send the
@@ -1086,9 +1230,10 @@ establish a new connection. If L</raise_error> disabled, the module will pass
 L<RedisDB::Error::DISCONNECTED> object to all outstanding callbacks and will
 try to reconnect to the server; it will also automatically restore
 subscriptions if object was in subscription mode. Module never tries to
-reconnect after MULTI command was sent to server and before corresponding EXEC
-or DISCARD was sent as this may cause data corruption, so during transaction
-module behaves like if L</raise_error> is set.
+reconnect after MULTI or WATCH command was sent to server and before
+corresponding UNWATCH, EXEC or DISCARD was sent as this may cause data
+corruption, so during transaction module behaves like if L</raise_error> is
+set.
 
 Module makes several attempts to reconnect each time increasing interval before
 the next attempt, depending on the values of L</reconnect_attempts> and
@@ -1307,7 +1452,7 @@ sub subscription_loop {
     return;
 }
 
-=head2 $self->subscribe($channel[, $callback])
+=head2 $self->subscribe($channel[, \&callback])
 
 Subscribe to the I<$channel>. If I<$callback> is not specified, default
 callback will be used in subscription loop, or messages will be returned by
@@ -1335,7 +1480,7 @@ sub subscribe {
     return;
 }
 
-=head2 $self->psubscribe($pattern[, $callback])
+=head2 $self->psubscribe($pattern[, \&callback])
 
 Subscribe to channels matching I<$pattern>. If I<$callback> is not specified,
 default callback will be used in subscription loop, or messages will be
@@ -1457,6 +1602,44 @@ exec will return false value.
 
 =cut
 
+=head2 $self->watch(@keys[, \&callback])
+
+mark given keys to be watched
+
+=cut
+
+sub watch {
+    my $self = shift;
+
+    $self->{_watching} = 1;
+    if ( ref $_[-1] eq 'CODE' ) {
+        return $self->send_command( 'WATCH', @_ );
+    }
+    else {
+        return $self->execute( 'WATCH', @_ );
+    }
+}
+
+=head2 $self->unwatch([\&callback])
+
+unwatch all keys
+
+=cut
+
+sub unwatch {
+    my $self = shift;
+
+    my $res;
+    if ( ref $_[-1] eq 'CODE' ) {
+        $res = $self->send_command( 'UNWATCH', @_ );
+    }
+    else {
+        $res = $self->execute( 'UNWATCH', @_ );
+    }
+    $self->{_watching} = undef;
+    return $res;
+}
+
 =head2 $self->multi([\&callback])
 
 Enter the transaction. After this and till I<exec> or I<discard> will be called,
@@ -1497,6 +1680,7 @@ sub exec {
         $res = $self->execute('EXEC');
     }
     $self->{_in_multi} = undef;
+    $self->{_watching} = undef;
     return $res;
 }
 
@@ -1517,9 +1701,20 @@ sub discard {
         $res = $self->execute('DISCARD');
     }
     $self->{_in_multi} = undef;
+    $self->{_watching} = undef;
     return $res;
 }
 
+=head1 CLUSTER SUPPORT
+
+For accessing redis cluster use L<RedisDB::Cluster> package
+
+=head1 SENTINEL SUPPORT
+
+For accessing redis servers managed by sentinel use L<RedisDB::Sentinel> package
+
+=cut
+
 1;
 
 __END__
@@ -1563,7 +1758,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011-2014 Pavel Shaydo.
+Copyright 2011-2015 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
