@@ -1,8 +1,8 @@
 package RedisDB::Error;
 
 use strict;
-use warnings;
-our $VERSION = "2.34_01";
+use base 'RedisDB::Parser::Error';
+our $VERSION = "2.48_01";
 $VERSION = eval $VERSION;
 
 =head1 NAME
@@ -21,34 +21,12 @@ RedisDB::Error - Error class for RedisDB
 
 Object of this class maybe passed as argument to callback specified in
 I<send_command_cb> if redis server return error.  In string context object
-returns description of the error.
-
-=head1 METHODS
+returns description of the error. This class inherits from
+L<RedisDB::Parser::Error>.
 
 =cut
 
-use overload '""' => \&as_string;
-
-=head2 $class->new($message)
-
-Create new error object with specified error message.
-
-=cut
-
-sub new {
-    my ($class, $message) = @_;
-    return bless { message => $message }, $class;
-}
-
-=head2 $self->as_string
-
-Return error message. Also you can just use object in string context.
-
-=cut
-
-sub as_string {
-    return shift->{message};
-}
+*new = \&RedisDB::Parser::Error::new;
 
 package RedisDB::Error::EAGAIN;
 our @ISA = qw(RedisDB::Error);
@@ -56,6 +34,12 @@ our @ISA = qw(RedisDB::Error);
 package RedisDB::Error::DISCONNECTED;
 our @ISA = qw(RedisDB::Error);
 
+package RedisDB::Error::MOVED;
+our @ISA = qw(RedisDB::Parser::Error::MOVED RedisDB::Error);
+
+package RedisDB::Error::ASK;
+our @ISA = qw(RedisDB::Parser::Error::ASK RedisDB::Error);
+
 1;
 
 __END__
@@ -70,7 +54,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011-2014 Pavel Shaydo.
+Copyright 2011-2015 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
